<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\EventListener\ORM;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\TypeAwareInterface;
use Dedi\SyliusCustomerPricingPlugin\Enum\ChannelPricingType;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\Event\LoadClassMetadataEventArgs;
use Doctrine\ORM\Events;
use Doctrine\ORM\Mapping\ClassMetadataInfo;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Resource\Metadata\RegistryInterface;

class ChannelPricingMetadataListener implements EventSubscriber
{
    public function __construct(private readonly RegistryInterface $registry)
    {
    }

    public function getSubscribedEvents()
    {
        return [
            Events::loadClassMetadata,
        ];
    }

    public function loadClassMetadata(LoadClassMetadataEventArgs $eventArgs)
    {
        $classMetadata = $eventArgs->getClassMetadata();
        $reflection = $classMetadata->getReflectionClass();

        if (!$reflection->implementsInterface(ChannelPricingInterface::class)) {
            return;
        }
        if ($reflection->isAbstract()) {
            return;
        }
        $defaultChannelPricingModel = $this->registry->get('sylius.channel_pricing')->getParameter('classes')['model'];
        if ($defaultChannelPricingModel !== $classMetadata->getName()) {
            return;
        }

        $type = !$reflection->implementsInterface(TypeAwareInterface::class) ? ChannelPricingType::DEFAULT->value : $reflection->getMethod('getType')->invoke(null);
        if (ChannelPricingType::DEFAULT->value !== $type) {
            return;
        }

        $classMetadata->setInheritanceType(ClassMetadataInfo::INHERITANCE_TYPE_JOINED);
        $classMetadata->setDiscriminatorColumn([
            'name' => 'type',
            'type' => 'string',
        ]);

        $classMetadata->addDiscriminatorMapClass(ChannelPricingType::DEFAULT->value, $defaultChannelPricingModel);
        $classMetadata->addDiscriminatorMapClass(ChannelPricingType::CUSTOMER->value, $this->registry->get('dedi_sylius_customer_pricing_plugin.customer_channel_pricing')->getParameter('classes')['model']);
        $classMetadata->addDiscriminatorMapClass(ChannelPricingType::ORGANIZATION->value, $this->registry->get('dedi_sylius_customer_pricing_plugin.organization_channel_pricing')->getParameter('classes')['model']);
        $classMetadata->addDiscriminatorMapClass(ChannelPricingType::CUSTOMER_GROUP->value, $this->registry->get('dedi_sylius_customer_pricing_plugin.customer_group_channel_pricing')->getParameter('classes')['model']);

        $table = $classMetadata->table;
        $table['uniqueConstraints'] = [];
        $classMetadata->setPrimaryTable($table);
    }
}
