<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Fixtures\Factory;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Customer\CustomerChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\CustomerGroup\CustomerGroupChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Organization\OrganizationChannelPricingInterface;
use InvalidArgumentException;
use Sylius\Bundle\CoreBundle\Fixture\Factory\ExampleFactoryInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Resource\Metadata\RegistryInterface;

class ChannelPricingExampleFactory implements ExampleFactoryInterface
{
    public function __construct(
        private readonly RegistryInterface $registry,
    ) {
    }

    public function create(array $options = []): ChannelPricingInterface
    {
        /** @var ?ChannelPricingInterface $originalChannelPricing */
        $originalChannelPricing = $options['pricing'] ?? null;
        if (null === $originalChannelPricing) {
            throw new InvalidArgumentException('Original channel pricing is required');
        }

        if (array_key_exists('customer', $options)) {
            $class = $this->registry->get('dedi_sylius_customer_pricing_plugin.customer_channel_pricing')->getParameter('classes')['model'];
            /** @var CustomerChannelPricingInterface $channelPricing */
            $channelPricing = new $class();
            $channelPricing->setCustomer($options['customer']);
        } elseif (array_key_exists('customer_group', $options)) {
            $class = $this->registry->get('dedi_sylius_customer_pricing_plugin.customer_group_channel_pricing')->getParameter('classes')['model'];
            /** @var CustomerGroupChannelPricingInterface $channelPricing */
            $channelPricing = new $class();
            $channelPricing->setCustomerGroup($options['customer_group']);
        } elseif (array_key_exists('organization', $options)) {
            $class = $this->registry->get('dedi_sylius_customer_pricing_plugin.organization_channel_pricing')->getParameter('classes')['model'];
            /** @var OrganizationChannelPricingInterface $channelPricing */
            $channelPricing = new $class();
            $channelPricing->setOrganization($options['organization']);
        } else {
            throw new InvalidArgumentException('Invalid channel pricing type');
        }

        $originalPrice = $originalChannelPricing->getOriginalPrice() ?? $originalChannelPricing->getPrice();
        $discount = random_int(5, 75);

        $channelPricing->setPrice((int) round($originalPrice - ($originalPrice * $discount / 100)));
        $channelPricing->setOriginalPrice($originalPrice);
        $channelPricing->setChannelCode($originalChannelPricing->getChannelCode());
        $channelPricing->setProductVariant($originalChannelPricing->getProductVariant());

        return $channelPricing;
    }
}
